"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useImportConnections = void 0;
const react_1 = require("react");
const renderer_1 = require("@mongodb-js/connection-storage/renderer");
const fs_1 = require("fs");
const common_1 = require("./common");
const INITIAL_STATE = Object.freeze({
    ...common_1.COMMON_INITIAL_STATE,
    passphraseRequired: false,
    fileContents: '',
});
async function loadFile({ filename, passphrase, favoriteConnectionIds, }, deserializeConnections) {
    if (!filename) {
        return INITIAL_STATE;
    }
    try {
        const fileContents = await fs_1.promises.readFile(filename, 'utf8');
        const connectionList = [];
        const connections = await deserializeConnections({
            content: fileContents,
            options: {
                passphrase,
            },
        });
        for (const info of connections) {
            if (info.favorite?.name) {
                const isExistingFavorite = favoriteConnectionIds.includes(info.id);
                connectionList.push({
                    name: info.favorite.name,
                    id: info.id,
                    selected: !isExistingFavorite,
                    isExistingFavorite,
                });
            }
        }
        if (connectionList.length === 0) {
            throw new Error('File does not contain any connections');
        }
        return {
            fileContents,
            connectionList,
            error: '',
        };
    }
    catch (err) {
        return {
            fileContents: '',
            connectionList: [],
            error: err.message,
            ...(err.passphraseRequired && { passphraseRequired: true }),
        };
    }
}
function useImportConnections({ finish, favoriteConnections, open, trackingProps, }, importConnections = renderer_1.ConnectionStorage.importConnections.bind(renderer_1.ConnectionStorage), deserializeConnections = renderer_1.ConnectionStorage.deserializeConnections.bind(renderer_1.ConnectionStorage)) {
    const [state, setState] = (0, react_1.useState)(INITIAL_STATE);
    (0, react_1.useEffect)(() => setState(INITIAL_STATE), [open]);
    const { passphrase, filename, fileContents, connectionList } = state;
    const favoriteConnectionIds = favoriteConnections.map(({ id }) => id);
    (0, react_1.useEffect)(() => {
        setState((prevState) => ({
            ...prevState,
            connectionList: state.connectionList.map((conn) => ({
                ...conn,
                isExistingFavorite: favoriteConnectionIds.includes(conn.id),
            })),
        }));
    }, [favoriteConnectionIds.join(',')]);
    const { onChangeConnectionList, onChangePassphrase, onCancel } = (0, common_1.useImportExportConnectionsCommon)(setState, finish);
    const onSubmit = (0, react_1.useCallback)(() => {
        setState((prevState) => ({ ...prevState, inProgress: true }));
        void (async () => {
            const filterConnectionIds = connectionList
                .filter((x) => x.selected)
                .map((x) => x.id);
            try {
                await importConnections({
                    content: fileContents,
                    options: {
                        passphrase,
                        filterConnectionIds,
                        trackingProps,
                    },
                });
            }
            catch (err) {
                setState((prevState) => {
                    return {
                        ...prevState,
                        inProgress: false,
                        error: err.message,
                    };
                });
                return;
            }
            finish('succeeded');
        })();
    }, [connectionList, fileContents, passphrase, finish]);
    const LOAD_CONNECTIONS_FILE_DEBOUNCE_DELAY = 100;
    (0, react_1.useEffect)(() => {
        let timer;
        timer = setTimeout(() => {
            timer = undefined;
            void loadFile({ filename, passphrase, favoriteConnectionIds }, deserializeConnections).then((stateUpdate) => {
                setState((prevState) => {
                    if (filename === prevState.filename &&
                        passphrase === prevState.passphrase)
                        return { ...prevState, ...stateUpdate };
                    return prevState;
                });
            });
        }, LOAD_CONNECTIONS_FILE_DEBOUNCE_DELAY);
        return () => {
            if (timer !== undefined)
                clearTimeout(timer);
        };
    }, [filename, passphrase, favoriteConnectionIds.join(',')]);
    const onChangeFilename = (0, react_1.useCallback)((filename) => {
        setState((prevState) => ({
            ...prevState,
            filename,
            ...(filename !== prevState.filename && {
                error: '',
                passphraseRequired: false,
            }),
        }));
    }, []);
    return {
        onCancel,
        onSubmit,
        onChangeFilename,
        onChangePassphrase,
        onChangeConnectionList,
        state,
    };
}
exports.useImportConnections = useImportConnections;
//# sourceMappingURL=use-import.js.map